package io.micronaut.http.client.filter

import io.micronaut.context.ApplicationContext
import io.micronaut.http.HttpResponse
import io.micronaut.http.MutableHttpRequest
import io.micronaut.http.annotation.Controller
import io.micronaut.http.annotation.Filter
import io.micronaut.http.annotation.Get
import io.micronaut.http.annotation.QueryValue
import io.micronaut.http.client.annotation.Client
import io.micronaut.http.filter.ClientFilterChain
import io.micronaut.http.filter.HttpClientFilter
import io.micronaut.http.uri.UriBuilder
import io.micronaut.runtime.server.EmbeddedServer
import org.reactivestreams.Publisher
import spock.lang.AutoCleanup
import spock.lang.Issue
import spock.lang.Shared
import spock.lang.Specification

import javax.annotation.Nullable

@Issue("https://github.com/micronaut-projects/micronaut-core/issues/917")
class MutateRequestClientFilterSpec extends Specification {

    @Shared
    @AutoCleanup
    EmbeddedServer embeddedServer = ApplicationContext.run(EmbeddedServer)

    @Shared
    MyClient myClient = embeddedServer.applicationContext.getBean(MyClient)

    void "test mutate outgoing URI"() {
        expect:
        myClient.simple() == "xxxxxxxxxxx"
        myClient.withQuery("foo") == "fooxxxxxxxxxxx"
    }

    @Client("/filters/uri/test")
    static interface MyClient {
        @Get("/")
        String simple()

        @Get("/foo{?q}")
        String withQuery(@Nullable String q)
    }

    @Controller('/filters/uri/test')
    static class UriController {
        @Get
        String result(@QueryValue String signature) {
            signature
        }
        @Get('/foo')
        String query(@QueryValue String signature, @QueryValue String q) {
            q + signature
        }
    }

    @Filter('/filters/uri/**')
    static class MyFilter implements HttpClientFilter {

        @Override
        Publisher<? extends HttpResponse<?>> doFilter(MutableHttpRequest<?> request, ClientFilterChain chain) {
            UriBuilder builder = UriBuilder.of(request.getUri())
            builder.queryParam("signature", "xxxxxxxxxxx")
            return chain.proceed(request.uri(builder.build()))
        }
    }
}
